Package: TissueEnrich
Type: Package
Title: Tissue-specific gene enrichment analysis
Version: 1.31.0
Authors@R: c(person("Ashish Jain",
        email="jain.ashishjain1@gmail.com",
        role=c("aut", "cre")), person("Geetu Tuteja",
        email="geetu@iastate.edu",
        role=c("aut")))
Description: The TissueEnrich package is used to calculate enrichment of tissue-specific genes in a set of input genes. For example, the user can input the most highly expressed genes from RNA-Seq data, or gene co-expression modules to determine which tissue-specific genes are enriched in those datasets. Tissue-specific genes were defined by processing RNA-Seq data from the Human Protein Atlas (HPA) (Uhlén et al. 2015), GTEx (Ardlie et al. 2015), and mouse ENCODE (Shen et al. 2012) using the algorithm from the HPA (Uhlén et al. 2015).The hypergeometric test is being used to determine if the tissue-specific genes are enriched among the input genes. Along with tissue-specific gene enrichment, the TissueEnrich package can also be used to define tissue-specific genes from expression datasets provided by the user, which can then be used to calculate tissue-specific gene enrichments.
License: MIT + file LICENSE
VignetteBuilder: knitr
Depends: R (>= 3.5), ggplot2 (>= 2.2.1), SummarizedExperiment (>=
        1.6.5), GSEABase (>= 1.38.2)
Imports: dplyr (>= 0.7.3), tidyr (>= 0.8.0), stats
Suggests: knitr, rmarkdown, testthat
Encoding: UTF-8
RoxygenNote: 7.3.1
biocViews: GeneSetEnrichment, GeneExpression, Sequencing
git_url: https://git.bioconductor.org/packages/TissueEnrich
git_branch: devel
git_last_commit: 706448a
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-11-01 00:39:25 UTC; biocbuild
Author: Ashish Jain [aut, cre],
  Geetu Tuteja [aut]
Maintainer: Ashish Jain <jain.ashishjain1@gmail.com>
