import(ggplot2)
import(dplyr)
import(SummarizedExperiment)
importFrom(GSEABase,GeneSet,SymbolIdentifier,EntrezIdentifier)
importFrom(tidyr, gather)
importFrom(stats, setNames)
export(teEnrichment)
export(teEnrichmentCustom)
export(teGeneRetrieval)
